@@ -12,7 +12,7 @@ class authz_allow_hosts(BaseAuthzHandler):
        Check if Origin is on the Allowed Hosts List.
     """
 
-    def check_authorization(self, request: web.Request) -> bool:
+    async def check_authorization(self, request: web.Request) -> bool:
         origin = request.host if request.host else request.headers["origin"]
         for key in ALLOWED_HOSTS:
             if fnmatch.fnmatch(origin, key):
