@@ -16,13 +16,14 @@ AUTHORIZATION_MIDDLEWARES = (
 )
 
 AUTHENTICATION_BACKENDS = (
- #'navigator.auth.backends.TrocToken',
- #'navigator.auth.backends.TokenAuth',
  #'navigator.auth.backends.GoogleAuth',
  # 'navigator.auth.backends.OktaAuth',
  #'navigator.auth.backends.ADFSAuth',
  # 'navigator.auth.backends.AzureAuth',
  #'navigator.auth.backends.GithubAuth',
+ 'navigator_auth.backends.APIKeyAuth',
+ 'navigator_auth.backends.TokenAuth',
+ 'navigator_auth.backends.TrocToken',
  'navigator_auth.backends.BasicAuth',
  'navigator_auth.backends.DjangoAuth',
  'navigator_auth.backends.NoAuth',
